// Lobster: tools for biological sequence analysis
// (C) Copyright 2002-2003 Robert C. Edgar
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.

#include "lobster.h"
#include "MSA.h"
#include <math.h>

static bool LocalEq(double x, double y)
	{
	double diff = fabs(x-y);
	return diff < 1e-3;
	}

void ComparePairMap(const int iTestMapA[], const int iTestMapB[],
  const int iRefMapA[], const int iRefMapB[], int iLengthA, int iLengthB,
  double *ptrdSP, double *ptrdPS, double *ptrdCS)
	{
	double dSP = SumPairs(iRefMapA, iTestMapA, iLengthA);
	double dPS = SumPairs(iTestMapA, iRefMapA, iLengthA);
	double dCS = ClineShift(iTestMapA, iRefMapA, iLengthA, iTestMapB,
	  iRefMapB, iLengthB);

#if	_DEBUG
	{
// Verify symmetries as a correctness check
	double dSP2 = SumPairs(iRefMapB, iTestMapB, iLengthB);
	double dPS2 = SumPairs(iTestMapB, iRefMapB, iLengthB);

	double dCS2 = ClineShift(iTestMapB, iRefMapB, iLengthB, iTestMapA,
	  iRefMapA, iLengthA);
	double dCS3 = ClineShift(iRefMapA, iTestMapA, iLengthA, iRefMapB,
	  iTestMapB, iLengthB);
	double dCS4 = ClineShift(iRefMapB, iTestMapB, iLengthB, iRefMapA,
	  iTestMapA, iLengthA);

	if (!LocalEq(dSP, dSP2))
		Quit("CompareSeqs: dSP=%g dSP2=%g diff=%g", dSP, dSP2, fabs(dSP-dSP2));
	if (!LocalEq(dPS, dPS2))
		Quit("CompareSeqs: dPS=%g dPS2=%g diff=%g", dPS, dPS2, fabs(dPS-dPS2));
	if (!LocalEq(dCS, dCS2))
		Quit("CompareSeqs: dCS=%g dCS2=%g diff=%g", dCS, dCS2, fabs(dCS-dCS2));
	if (!LocalEq(dCS, dCS3))
		Quit("CompareSeqs: dCS=%g dCS3=%g diff=%g", dCS, dCS3, fabs(dCS-dCS3));
	if (!LocalEq(dCS, dCS4))
		Quit("CompareSeqs: dCS=%g dCS4=%g diff=%g", dCS, dCS4, fabs(dCS-dCS4));
	}
#endif

	*ptrdSP = dSP;
	*ptrdPS = dPS;
	*ptrdCS = dCS;
	}
