// Lobster: tools for biological sequence analysis
// (C) Copyright 2002-2003 Robert C. Edgar
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.

#include "lobster.h"
#include "MSA.h"
#include "TextFile.h"

void DoCompareAlns(const char *test, const char *ref)
	{
	const char *strTestFileName = test;
	const char *strRefFileName = ref;

	TextFile TestFile(strTestFileName);
	MSA msaTest;
	msaTest.FromFile(TestFile);
	msaTest.AlignByCase();

	TextFile RefFile(strRefFileName);
	MSA msaRef;
	msaRef.FromFile(RefFile);
	msaRef.AlignByCase();

	double dSP = dInsane;
	double dTC = dInsane;
	double dPS = dInsane;
	BaliScore(msaRef, msaTest, &dSP, &dTC, &dPS);

	double dCS = dInsane;
	ClineScore(msaRef, msaTest, 0.2, &dCS);

	char szTestName[256];
	char szRefName[256];

	NameFromPath(strTestFileName, szTestName, sizeof(szTestName));
	NameFromPath(strRefFileName, szRefName, sizeof(szRefName));

	List("Test=%s;Ref=%s;SP=%.3g;TC=%.3g;CS=%.3g;PS=%.3g\n",
		szTestName,
		szRefName,
		dSP,
		dTC,
		dCS,
		dPS);

	printf("Test=%s;Ref=%s;SP=%.3g;TC=%.3g;CS=%.3g;PS=%.3g\n",
		szTestName,
		szRefName,
		dSP,
		dTC,
		dCS,
		dPS);
	}
