// Lobster: tools for biological sequence analysis
// (C) Copyright 2002-2003 Robert C. Edgar
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.

#include "lobster.h"
#include "SatchmoParams.h"
#include "TextFile.h"
#include "SeqVect.h"
#include "GTree2.h"
#include "GNode2.h"
#include "SJPM.h"

TextFile *g_ptrSmoFile;

void DoSatchmo(const char *in)
	{
	const char *strSatchmoInputFileName = in;

// Three kinds of output -- must specify at least one of them
	const char *strOutputFastaFileName = GetOptionalParam("out");
	const char *strOutputSVFileName = GetOptionalParam("sv");
	const char *strPhylipFileName = GetOptionalParam("phy");

	if (0 == strOutputFastaFileName && 0 == strPhylipFileName &&
	  0 == strOutputSVFileName)
		CmdLineError("No output file specified for Satchmo");

	unsigned uWindowLength = 7;				// TODO
	double dMinAvgAff = -9.0;				// TODO
	double dMaxNIC = 3;						// TODO

	if (0 == uWindowLength%2)
		CmdLineError("Window length %u, must be an odd integer.");

	SatchmoParams Params;
	Params.SetMinAvgAff(dMinAvgAff);
	Params.SetWindowLength(uWindowLength);
	Params.SetMaxNIC(dMaxNIC);

	TextFile InputFile(strSatchmoInputFileName);

	GTree2 Tree;
	SeqVect Seqs;
	Seqs.FromFASTAFile(InputFile);

	Tree.SetInputSeqs(Seqs);

	if (0 != strOutputSVFileName)
		g_ptrSmoFile = new TextFile(strOutputSVFileName, true);

	BuildMonitor BM;
	Tree.Build(Params, &BM);
	if (0 != g_ptrSmoFile)
		{
		delete g_ptrSmoFile;
		g_ptrSmoFile = 0;
		}

	GNode2 *ptrRoot = Tree.GetRoot();
	if (0 != ptrRoot)
		{
		const MSA &a = ptrRoot->m_MSA;
		if (0 != strOutputFastaFileName)
			{
			TextFile File(strOutputFastaFileName, true);
			a.ToFASTAFile(File);
			}
		if (0 != strPhylipFileName)
			{
			TextFile File(strPhylipFileName, true);
			Tree.ToPhylipFile(File);
			}
		}
	else
		{
		fprintf(stderr, "No root, tree not completed.\n");
		List("No root, tree not completed.\n");
		}
	}
