// Lobster: tools for biological sequence analysis
// (C) Copyright 2002-2003 Robert C. Edgar
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.

#ifndef	HMMPath_h
#define HMMPath_h

class Seq;
class MSA;
class SatchmoParams;
class HMM;
class TextFile;

class HMMEdge
	{
public:
	char cState;
	unsigned uNodeIndex;
	unsigned uPrefixLength;
	SCORE scoreEmit;
	};

class HMMPath
	{
// Disable compiler defaults
private:
	HMMPath &operator=(const HMMPath &rhs);
	HMMPath(const HMMPath &rhs);

public:
	HMMPath();
	virtual ~HMMPath();

public:
	void Clear();
	void Copy(const HMMPath &Path);
	void AppendEdge(const HMMEdge &Edge);
	void PrependEdge(const HMMEdge &Edge);
	unsigned GetEdgeCount() const { return m_uEdgeCount; }
	const HMMEdge &GetEdge(unsigned uEdgeIndex) const;
	void Validate(unsigned uNodeCount, unsigned uPrefixCount, MODEL_BOUNDS ModelBounds,
	  SEQ_BOUNDS SeqBounds) const;
	void ListMe() const;
	SCORE GetAvgAff(unsigned uEdgeIndex, unsigned uWindowLength) const;
	bool Eq(const HMMPath &Path) const;
	SCORE GetEdgeScore(unsigned uEdgeIndex, const HMM &Model, const Seq &s) const;
	SCORE GetTransScore(unsigned uEdgeIndex, const HMM &Model) const;
	unsigned GetEmitterCount() const;
	void ToFile(TextFile &File) const;
	void FromFile(TextFile &File);

private:
	void ExpandPath(unsigned uAdditionalEdgeCount);

private:
	unsigned m_uEdgeCount;
	unsigned m_uArraySize;
	HMMEdge *m_Edges;
	};

#endif	// HMMPath_h
