// Lobster: tools for biological sequence analysis
// (C) Copyright 2002-2003 Robert C. Edgar
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.

#include "lobster.h"
#include "PWLA.h"
#include "PWLAI.h"
#include <math.h>

// Log average score.
// von Ohsen, N., Sommer, I., Zimmer, R., "Profile-profile alignment:
// a powerful tool for protein structure prediction", Pacific Symposium
// on Biocomputing 2003, 252-263.
SCORE PWLA::LAScore(const FCOUNT fcCountsA[MAX_ALPHA],
  const FCOUNT fcCountsB[MAX_ALPHA])
	{
	double dSum = 0.0;

// The following code is a loop-unrolled version of this:
//	for (unsigned uLetterA = 0; uLetterA < MAX_ALPHA; ++uLetterA)
//		for (unsigned uLetterB = 0; uLetterB < MAX_ALPHA; ++uLetterB)
//			dSum += fcCountsA[uLetterA]*fcCountsB[uLetterB]*p_rel(i, j) / (p_i p_j);
	dSum += fcCountsA[0]*fcCountsB[0]*3.9029401;
	dSum += fcCountsA[0]*fcCountsB[1]*0.8679881;
	dSum += fcCountsA[0]*fcCountsB[2]*0.5446049;
	dSum += fcCountsA[0]*fcCountsB[3]*0.7412640;
	dSum += fcCountsA[0]*fcCountsB[4]*0.4648942;
	dSum += fcCountsA[0]*fcCountsB[5]*1.0568696;
	dSum += fcCountsA[0]*fcCountsB[6]*0.5693654;
	dSum += fcCountsA[0]*fcCountsB[7]*0.6324813;
	dSum += fcCountsA[0]*fcCountsB[8]*0.7753898;
	dSum += fcCountsA[0]*fcCountsB[9]*0.6019460;
	dSum += fcCountsA[0]*fcCountsB[10]*0.7231498;
	dSum += fcCountsA[0]*fcCountsB[11]*0.5883077;
	dSum += fcCountsA[0]*fcCountsB[12]*0.7541214;
	dSum += fcCountsA[0]*fcCountsB[13]*0.7568035;
	dSum += fcCountsA[0]*fcCountsB[14]*0.6126988;
	dSum += fcCountsA[0]*fcCountsB[15]*1.4721037;
	dSum += fcCountsA[0]*fcCountsB[16]*0.9844022;
	dSum += fcCountsA[0]*fcCountsB[17]*0.9364584;
	dSum += fcCountsA[0]*fcCountsB[18]*0.4165484;
	dSum += fcCountsA[0]*fcCountsB[19]*0.5426125;
	dSum += fcCountsA[1]*fcCountsB[0]*0.8679881;
	dSum += fcCountsA[1]*fcCountsB[1]*19.5765802;
	dSum += fcCountsA[1]*fcCountsB[2]*0.3014542;
	dSum += fcCountsA[1]*fcCountsB[3]*0.2859347;
	dSum += fcCountsA[1]*fcCountsB[4]*0.4389910;
	dSum += fcCountsA[1]*fcCountsB[5]*0.4203886;
	dSum += fcCountsA[1]*fcCountsB[6]*0.3550472;
	dSum += fcCountsA[1]*fcCountsB[7]*0.6534589;
	dSum += fcCountsA[1]*fcCountsB[8]*0.3491296;
	dSum += fcCountsA[1]*fcCountsB[9]*0.6422760;
	dSum += fcCountsA[1]*fcCountsB[10]*0.6113537;
	dSum += fcCountsA[1]*fcCountsB[11]*0.3978026;
	dSum += fcCountsA[1]*fcCountsB[12]*0.3795628;
	dSum += fcCountsA[1]*fcCountsB[13]*0.3657796;
	dSum += fcCountsA[1]*fcCountsB[14]*0.3089379;
	dSum += fcCountsA[1]*fcCountsB[15]*0.7384148;
	dSum += fcCountsA[1]*fcCountsB[16]*0.7405530;
	dSum += fcCountsA[1]*fcCountsB[17]*0.7558448;
	dSum += fcCountsA[1]*fcCountsB[18]*0.4499807;
	dSum += fcCountsA[1]*fcCountsB[19]*0.4342013;
	dSum += fcCountsA[2]*fcCountsB[0]*0.5446049;
	dSum += fcCountsA[2]*fcCountsB[1]*0.3014542;
	dSum += fcCountsA[2]*fcCountsB[2]*7.3979253;
	dSum += fcCountsA[2]*fcCountsB[3]*1.6878109;
	dSum += fcCountsA[2]*fcCountsB[4]*0.2989696;
	dSum += fcCountsA[2]*fcCountsB[5]*0.6343015;
	dSum += fcCountsA[2]*fcCountsB[6]*0.6785593;
	dSum += fcCountsA[2]*fcCountsB[7]*0.3390155;
	dSum += fcCountsA[2]*fcCountsB[8]*0.7840905;
	dSum += fcCountsA[2]*fcCountsB[9]*0.2866128;
	dSum += fcCountsA[2]*fcCountsB[10]*0.3464547;
	dSum += fcCountsA[2]*fcCountsB[11]*1.5538520;
	dSum += fcCountsA[2]*fcCountsB[12]*0.5987177;
	dSum += fcCountsA[2]*fcCountsB[13]*0.8970811;
	dSum += fcCountsA[2]*fcCountsB[14]*0.5732000;
	dSum += fcCountsA[2]*fcCountsB[15]*0.9135051;
	dSum += fcCountsA[2]*fcCountsB[16]*0.6947898;
	dSum += fcCountsA[2]*fcCountsB[17]*0.3365004;
	dSum += fcCountsA[2]*fcCountsB[18]*0.2321050;
	dSum += fcCountsA[2]*fcCountsB[19]*0.3456829;
	dSum += fcCountsA[3]*fcCountsB[0]*0.7412640;
	dSum += fcCountsA[3]*fcCountsB[1]*0.2859347;
	dSum += fcCountsA[3]*fcCountsB[2]*1.6878109;
	dSum += fcCountsA[3]*fcCountsB[3]*5.4695276;
	dSum += fcCountsA[3]*fcCountsB[4]*0.3307441;
	dSum += fcCountsA[3]*fcCountsB[5]*0.4812675;
	dSum += fcCountsA[3]*fcCountsB[6]*0.9600400;
	dSum += fcCountsA[3]*fcCountsB[7]*0.3305223;
	dSum += fcCountsA[3]*fcCountsB[8]*1.3082782;
	dSum += fcCountsA[3]*fcCountsB[9]*0.3728734;
	dSum += fcCountsA[3]*fcCountsB[10]*0.5003421;
	dSum += fcCountsA[3]*fcCountsB[11]*0.9112983;
	dSum += fcCountsA[3]*fcCountsB[12]*0.6792027;
	dSum += fcCountsA[3]*fcCountsB[13]*1.9017376;
	dSum += fcCountsA[3]*fcCountsB[14]*0.9607983;
	dSum += fcCountsA[3]*fcCountsB[15]*0.9503570;
	dSum += fcCountsA[3]*fcCountsB[16]*0.7414260;
	dSum += fcCountsA[3]*fcCountsB[17]*0.4289431;
	dSum += fcCountsA[3]*fcCountsB[18]*0.3743021;
	dSum += fcCountsA[3]*fcCountsB[19]*0.4964664;
	dSum += fcCountsA[4]*fcCountsB[0]*0.4648942;
	dSum += fcCountsA[4]*fcCountsB[1]*0.4389910;
	dSum += fcCountsA[4]*fcCountsB[2]*0.2989696;
	dSum += fcCountsA[4]*fcCountsB[3]*0.3307441;
	dSum += fcCountsA[4]*fcCountsB[4]*8.1287983;
	dSum += fcCountsA[4]*fcCountsB[5]*0.3406407;
	dSum += fcCountsA[4]*fcCountsB[6]*0.6519893;
	dSum += fcCountsA[4]*fcCountsB[7]*0.9457698;
	dSum += fcCountsA[4]*fcCountsB[8]*0.3440433;
	dSum += fcCountsA[4]*fcCountsB[9]*1.1545978;
	dSum += fcCountsA[4]*fcCountsB[10]*1.0043715;
	dSum += fcCountsA[4]*fcCountsB[11]*0.3542882;
	dSum += fcCountsA[4]*fcCountsB[12]*0.2874440;
	dSum += fcCountsA[4]*fcCountsB[13]*0.3339729;
	dSum += fcCountsA[4]*fcCountsB[14]*0.3807263;
	dSum += fcCountsA[4]*fcCountsB[15]*0.4399736;
	dSum += fcCountsA[4]*fcCountsB[16]*0.4816930;
	dSum += fcCountsA[4]*fcCountsB[17]*0.7450894;
	dSum += fcCountsA[4]*fcCountsB[18]*1.3743775;
	dSum += fcCountsA[4]*fcCountsB[19]*2.7693817;
	dSum += fcCountsA[5]*fcCountsB[0]*1.0568696;
	dSum += fcCountsA[5]*fcCountsB[1]*0.4203886;
	dSum += fcCountsA[5]*fcCountsB[2]*0.6343015;
	dSum += fcCountsA[5]*fcCountsB[3]*0.4812675;
	dSum += fcCountsA[5]*fcCountsB[4]*0.3406407;
	dSum += fcCountsA[5]*fcCountsB[5]*6.8763075;
	dSum += fcCountsA[5]*fcCountsB[6]*0.4929663;
	dSum += fcCountsA[5]*fcCountsB[7]*0.2750096;
	dSum += fcCountsA[5]*fcCountsB[8]*0.5888716;
	dSum += fcCountsA[5]*fcCountsB[9]*0.2845039;
	dSum += fcCountsA[5]*fcCountsB[10]*0.3954865;
	dSum += fcCountsA[5]*fcCountsB[11]*0.8637114;
	dSum += fcCountsA[5]*fcCountsB[12]*0.4773858;
	dSum += fcCountsA[5]*fcCountsB[13]*0.5386498;
	dSum += fcCountsA[5]*fcCountsB[14]*0.4499840;
	dSum += fcCountsA[5]*fcCountsB[15]*0.9035965;
	dSum += fcCountsA[5]*fcCountsB[16]*0.5792712;
	dSum += fcCountsA[5]*fcCountsB[17]*0.3369551;
	dSum += fcCountsA[5]*fcCountsB[18]*0.4216898;
	dSum += fcCountsA[5]*fcCountsB[19]*0.3487141;
	dSum += fcCountsA[6]*fcCountsB[0]*0.5693654;
	dSum += fcCountsA[6]*fcCountsB[1]*0.3550472;
	dSum += fcCountsA[6]*fcCountsB[2]*0.6785593;
	dSum += fcCountsA[6]*fcCountsB[3]*0.9600400;
	dSum += fcCountsA[6]*fcCountsB[4]*0.6519893;
	dSum += fcCountsA[6]*fcCountsB[5]*0.4929663;
	dSum += fcCountsA[6]*fcCountsB[6]*13.5060070;
	dSum += fcCountsA[6]*fcCountsB[7]*0.3262878;
	dSum += fcCountsA[6]*fcCountsB[8]*0.7788884;
	dSum += fcCountsA[6]*fcCountsB[9]*0.3806759;
	dSum += fcCountsA[6]*fcCountsB[10]*0.5841316;
	dSum += fcCountsA[6]*fcCountsB[11]*1.2220028;
	dSum += fcCountsA[6]*fcCountsB[12]*0.4728797;
	dSum += fcCountsA[6]*fcCountsB[13]*1.1679835;
	dSum += fcCountsA[6]*fcCountsB[14]*0.9170473;
	dSum += fcCountsA[6]*fcCountsB[15]*0.7367319;
	dSum += fcCountsA[6]*fcCountsB[16]*0.5575021;
	dSum += fcCountsA[6]*fcCountsB[17]*0.3394474;
	dSum += fcCountsA[6]*fcCountsB[18]*0.4440859;
	dSum += fcCountsA[6]*fcCountsB[19]*1.7979036;
	dSum += fcCountsA[7]*fcCountsB[0]*0.6324813;
	dSum += fcCountsA[7]*fcCountsB[1]*0.6534589;
	dSum += fcCountsA[7]*fcCountsB[2]*0.3390155;
	dSum += fcCountsA[7]*fcCountsB[3]*0.3305223;
	dSum += fcCountsA[7]*fcCountsB[4]*0.9457698;
	dSum += fcCountsA[7]*fcCountsB[5]*0.2750096;
	dSum += fcCountsA[7]*fcCountsB[6]*0.3262878;
	dSum += fcCountsA[7]*fcCountsB[7]*3.9979299;
	dSum += fcCountsA[7]*fcCountsB[8]*0.3963730;
	dSum += fcCountsA[7]*fcCountsB[9]*1.6944349;
	dSum += fcCountsA[7]*fcCountsB[10]*1.4777449;
	dSum += fcCountsA[7]*fcCountsB[11]*0.3279345;
	dSum += fcCountsA[7]*fcCountsB[12]*0.3846629;
	dSum += fcCountsA[7]*fcCountsB[13]*0.3829375;
	dSum += fcCountsA[7]*fcCountsB[14]*0.3547509;
	dSum += fcCountsA[7]*fcCountsB[15]*0.4431634;
	dSum += fcCountsA[7]*fcCountsB[16]*0.7798163;
	dSum += fcCountsA[7]*fcCountsB[17]*2.4175121;
	dSum += fcCountsA[7]*fcCountsB[18]*0.4088732;
	dSum += fcCountsA[7]*fcCountsB[19]*0.6303898;
	dSum += fcCountsA[8]*fcCountsB[0]*0.7753898;
	dSum += fcCountsA[8]*fcCountsB[1]*0.3491296;
	dSum += fcCountsA[8]*fcCountsB[2]*0.7840905;
	dSum += fcCountsA[8]*fcCountsB[3]*1.3082782;
	dSum += fcCountsA[8]*fcCountsB[4]*0.3440433;
	dSum += fcCountsA[8]*fcCountsB[5]*0.5888716;
	dSum += fcCountsA[8]*fcCountsB[6]*0.7788884;
	dSum += fcCountsA[8]*fcCountsB[7]*0.3963730;
	dSum += fcCountsA[8]*fcCountsB[8]*4.7643359;
	dSum += fcCountsA[8]*fcCountsB[9]*0.4282702;
	dSum += fcCountsA[8]*fcCountsB[10]*0.6253033;
	dSum += fcCountsA[8]*fcCountsB[11]*0.9398419;
	dSum += fcCountsA[8]*fcCountsB[12]*0.7037741;
	dSum += fcCountsA[8]*fcCountsB[13]*1.5543233;
	dSum += fcCountsA[8]*fcCountsB[14]*2.0768092;
	dSum += fcCountsA[8]*fcCountsB[15]*0.9319192;
	dSum += fcCountsA[8]*fcCountsB[16]*0.7929060;
	dSum += fcCountsA[8]*fcCountsB[17]*0.4565429;
	dSum += fcCountsA[8]*fcCountsB[18]*0.3589319;
	dSum += fcCountsA[8]*fcCountsB[19]*0.5321784;
	dSum += fcCountsA[9]*fcCountsB[0]*0.6019460;
	dSum += fcCountsA[9]*fcCountsB[1]*0.6422760;
	dSum += fcCountsA[9]*fcCountsB[2]*0.2866128;
	dSum += fcCountsA[9]*fcCountsB[3]*0.3728734;
	dSum += fcCountsA[9]*fcCountsB[4]*1.1545978;
	dSum += fcCountsA[9]*fcCountsB[5]*0.2845039;
	dSum += fcCountsA[9]*fcCountsB[6]*0.3806759;
	dSum += fcCountsA[9]*fcCountsB[7]*1.6944349;
	dSum += fcCountsA[9]*fcCountsB[8]*0.4282702;
	dSum += fcCountsA[9]*fcCountsB[9]*3.7966214;
	dSum += fcCountsA[9]*fcCountsB[10]*1.9942957;
	dSum += fcCountsA[9]*fcCountsB[11]*0.3100430;
	dSum += fcCountsA[9]*fcCountsB[12]*0.3711219;
	dSum += fcCountsA[9]*fcCountsB[13]*0.4773261;
	dSum += fcCountsA[9]*fcCountsB[14]*0.4739194;
	dSum += fcCountsA[9]*fcCountsB[15]*0.4288939;
	dSum += fcCountsA[9]*fcCountsB[16]*0.6603292;
	dSum += fcCountsA[9]*fcCountsB[17]*1.3142355;
	dSum += fcCountsA[9]*fcCountsB[18]*0.5680359;
	dSum += fcCountsA[9]*fcCountsB[19]*0.6920589;
	dSum += fcCountsA[10]*fcCountsB[0]*0.7231498;
	dSum += fcCountsA[10]*fcCountsB[1]*0.6113537;
	dSum += fcCountsA[10]*fcCountsB[2]*0.3464547;
	dSum += fcCountsA[10]*fcCountsB[3]*0.5003421;
	dSum += fcCountsA[10]*fcCountsB[4]*1.0043715;
	dSum += fcCountsA[10]*fcCountsB[5]*0.3954865;
	dSum += fcCountsA[10]*fcCountsB[6]*0.5841316;
	dSum += fcCountsA[10]*fcCountsB[7]*1.4777449;
	dSum += fcCountsA[10]*fcCountsB[8]*0.6253033;
	dSum += fcCountsA[10]*fcCountsB[9]*1.9942957;
	dSum += fcCountsA[10]*fcCountsB[10]*6.4814549;
	dSum += fcCountsA[10]*fcCountsB[11]*0.4745299;
	dSum += fcCountsA[10]*fcCountsB[12]*0.4238960;
	dSum += fcCountsA[10]*fcCountsB[13]*0.8642486;
	dSum += fcCountsA[10]*fcCountsB[14]*0.6226249;
	dSum += fcCountsA[10]*fcCountsB[15]*0.5985578;
	dSum += fcCountsA[10]*fcCountsB[16]*0.7938018;
	dSum += fcCountsA[10]*fcCountsB[17]*1.2689365;
	dSum += fcCountsA[10]*fcCountsB[18]*0.6103022;
	dSum += fcCountsA[10]*fcCountsB[19]*0.7083636;
	dSum += fcCountsA[11]*fcCountsB[0]*0.5883077;
	dSum += fcCountsA[11]*fcCountsB[1]*0.3978026;
	dSum += fcCountsA[11]*fcCountsB[2]*1.5538520;
	dSum += fcCountsA[11]*fcCountsB[3]*0.9112983;
	dSum += fcCountsA[11]*fcCountsB[4]*0.3542882;
	dSum += fcCountsA[11]*fcCountsB[5]*0.8637114;
	dSum += fcCountsA[11]*fcCountsB[6]*1.2220028;
	dSum += fcCountsA[11]*fcCountsB[7]*0.3279345;
	dSum += fcCountsA[11]*fcCountsB[8]*0.9398419;
	dSum += fcCountsA[11]*fcCountsB[9]*0.3100430;
	dSum += fcCountsA[11]*fcCountsB[10]*0.4745299;
	dSum += fcCountsA[11]*fcCountsB[11]*7.0940964;
	dSum += fcCountsA[11]*fcCountsB[12]*0.4999337;
	dSum += fcCountsA[11]*fcCountsB[13]*1.0005835;
	dSum += fcCountsA[11]*fcCountsB[14]*0.8586298;
	dSum += fcCountsA[11]*fcCountsB[15]*1.2315289;
	dSum += fcCountsA[11]*fcCountsB[16]*0.9841525;
	dSum += fcCountsA[11]*fcCountsB[17]*0.3690340;
	dSum += fcCountsA[11]*fcCountsB[18]*0.2777841;
	dSum += fcCountsA[11]*fcCountsB[19]*0.4860309;
	dSum += fcCountsA[12]*fcCountsB[0]*0.7541214;
	dSum += fcCountsA[12]*fcCountsB[1]*0.3795628;
	dSum += fcCountsA[12]*fcCountsB[2]*0.5987177;
	dSum += fcCountsA[12]*fcCountsB[3]*0.6792027;
	dSum += fcCountsA[12]*fcCountsB[4]*0.2874440;
	dSum += fcCountsA[12]*fcCountsB[5]*0.4773858;
	dSum += fcCountsA[12]*fcCountsB[6]*0.4728797;
	dSum += fcCountsA[12]*fcCountsB[7]*0.3846629;
	dSum += fcCountsA[12]*fcCountsB[8]*0.7037741;
	dSum += fcCountsA[12]*fcCountsB[9]*0.3711219;
	dSum += fcCountsA[12]*fcCountsB[10]*0.4238960;
	dSum += fcCountsA[12]*fcCountsB[11]*0.4999337;
	dSum += fcCountsA[12]*fcCountsB[12]*12.8375452;
	dSum += fcCountsA[12]*fcCountsB[13]*0.6412803;
	dSum += fcCountsA[12]*fcCountsB[14]*0.4815348;
	dSum += fcCountsA[12]*fcCountsB[15]*0.7555033;
	dSum += fcCountsA[12]*fcCountsB[16]*0.6888962;
	dSum += fcCountsA[12]*fcCountsB[17]*0.4430825;
	dSum += fcCountsA[12]*fcCountsB[18]*0.2818321;
	dSum += fcCountsA[12]*fcCountsB[19]*0.3635216;
	dSum += fcCountsA[13]*fcCountsB[0]*0.7568035;
	dSum += fcCountsA[13]*fcCountsB[1]*0.3657796;
	dSum += fcCountsA[13]*fcCountsB[2]*0.8970811;
	dSum += fcCountsA[13]*fcCountsB[3]*1.9017376;
	dSum += fcCountsA[13]*fcCountsB[4]*0.3339729;
	dSum += fcCountsA[13]*fcCountsB[5]*0.5386498;
	dSum += fcCountsA[13]*fcCountsB[6]*1.1679835;
	dSum += fcCountsA[13]*fcCountsB[7]*0.3829375;
	dSum += fcCountsA[13]*fcCountsB[8]*1.5543233;
	dSum += fcCountsA[13]*fcCountsB[9]*0.4773261;
	dSum += fcCountsA[13]*fcCountsB[10]*0.8642486;
	dSum += fcCountsA[13]*fcCountsB[11]*1.0005835;
	dSum += fcCountsA[13]*fcCountsB[12]*0.6412803;
	dSum += fcCountsA[13]*fcCountsB[13]*6.2444210;
	dSum += fcCountsA[13]*fcCountsB[14]*1.4057958;
	dSum += fcCountsA[13]*fcCountsB[15]*0.9655559;
	dSum += fcCountsA[13]*fcCountsB[16]*0.7913219;
	dSum += fcCountsA[13]*fcCountsB[17]*0.4667781;
	dSum += fcCountsA[13]*fcCountsB[18]*0.5093584;
	dSum += fcCountsA[13]*fcCountsB[19]*0.6110951;
	dSum += fcCountsA[14]*fcCountsB[0]*0.6126988;
	dSum += fcCountsA[14]*fcCountsB[1]*0.3089379;
	dSum += fcCountsA[14]*fcCountsB[2]*0.5732000;
	dSum += fcCountsA[14]*fcCountsB[3]*0.9607983;
	dSum += fcCountsA[14]*fcCountsB[4]*0.3807263;
	dSum += fcCountsA[14]*fcCountsB[5]*0.4499840;
	dSum += fcCountsA[14]*fcCountsB[6]*0.9170473;
	dSum += fcCountsA[14]*fcCountsB[7]*0.3547509;
	dSum += fcCountsA[14]*fcCountsB[8]*2.0768092;
	dSum += fcCountsA[14]*fcCountsB[9]*0.4739194;
	dSum += fcCountsA[14]*fcCountsB[10]*0.6226249;
	dSum += fcCountsA[14]*fcCountsB[11]*0.8586298;
	dSum += fcCountsA[14]*fcCountsB[12]*0.4815348;
	dSum += fcCountsA[14]*fcCountsB[13]*1.4057958;
	dSum += fcCountsA[14]*fcCountsB[14]*6.6655769;
	dSum += fcCountsA[14]*fcCountsB[15]*0.7671661;
	dSum += fcCountsA[14]*fcCountsB[16]*0.6777544;
	dSum += fcCountsA[14]*fcCountsB[17]*0.4200721;
	dSum += fcCountsA[14]*fcCountsB[18]*0.3951049;
	dSum += fcCountsA[14]*fcCountsB[19]*0.5559652;
	dSum += fcCountsA[15]*fcCountsB[0]*1.4721037;
	dSum += fcCountsA[15]*fcCountsB[1]*0.7384148;
	dSum += fcCountsA[15]*fcCountsB[2]*0.9135051;
	dSum += fcCountsA[15]*fcCountsB[3]*0.9503570;
	dSum += fcCountsA[15]*fcCountsB[4]*0.4399736;
	dSum += fcCountsA[15]*fcCountsB[5]*0.9035965;
	dSum += fcCountsA[15]*fcCountsB[6]*0.7367319;
	dSum += fcCountsA[15]*fcCountsB[7]*0.4431634;
	dSum += fcCountsA[15]*fcCountsB[8]*0.9319192;
	dSum += fcCountsA[15]*fcCountsB[9]*0.4288939;
	dSum += fcCountsA[15]*fcCountsB[10]*0.5985578;
	dSum += fcCountsA[15]*fcCountsB[11]*1.2315289;
	dSum += fcCountsA[15]*fcCountsB[12]*0.7555033;
	dSum += fcCountsA[15]*fcCountsB[13]*0.9655559;
	dSum += fcCountsA[15]*fcCountsB[14]*0.7671661;
	dSum += fcCountsA[15]*fcCountsB[15]*3.8428476;
	dSum += fcCountsA[15]*fcCountsB[16]*1.6139205;
	dSum += fcCountsA[15]*fcCountsB[17]*0.5652240;
	dSum += fcCountsA[15]*fcCountsB[18]*0.3853031;
	dSum += fcCountsA[15]*fcCountsB[19]*0.5575206;
	dSum += fcCountsA[16]*fcCountsB[0]*0.9844022;
	dSum += fcCountsA[16]*fcCountsB[1]*0.7405530;
	dSum += fcCountsA[16]*fcCountsB[2]*0.6947898;
	dSum += fcCountsA[16]*fcCountsB[3]*0.7414260;
	dSum += fcCountsA[16]*fcCountsB[4]*0.4816930;
	dSum += fcCountsA[16]*fcCountsB[5]*0.5792712;
	dSum += fcCountsA[16]*fcCountsB[6]*0.5575021;
	dSum += fcCountsA[16]*fcCountsB[7]*0.7798163;
	dSum += fcCountsA[16]*fcCountsB[8]*0.7929060;
	dSum += fcCountsA[16]*fcCountsB[9]*0.6603292;
	dSum += fcCountsA[16]*fcCountsB[10]*0.7938018;
	dSum += fcCountsA[16]*fcCountsB[11]*0.9841525;
	dSum += fcCountsA[16]*fcCountsB[12]*0.6888962;
	dSum += fcCountsA[16]*fcCountsB[13]*0.7913219;
	dSum += fcCountsA[16]*fcCountsB[14]*0.6777544;
	dSum += fcCountsA[16]*fcCountsB[15]*1.6139205;
	dSum += fcCountsA[16]*fcCountsB[16]*4.8321048;
	dSum += fcCountsA[16]*fcCountsB[17]*0.9809432;
	dSum += fcCountsA[16]*fcCountsB[18]*0.4309317;
	dSum += fcCountsA[16]*fcCountsB[19]*0.5731577;
	dSum += fcCountsA[17]*fcCountsB[0]*0.9364584;
	dSum += fcCountsA[17]*fcCountsB[1]*0.7558448;
	dSum += fcCountsA[17]*fcCountsB[2]*0.3365004;
	dSum += fcCountsA[17]*fcCountsB[3]*0.4289431;
	dSum += fcCountsA[17]*fcCountsB[4]*0.7450894;
	dSum += fcCountsA[17]*fcCountsB[5]*0.3369551;
	dSum += fcCountsA[17]*fcCountsB[6]*0.3394474;
	dSum += fcCountsA[17]*fcCountsB[7]*2.4175121;
	dSum += fcCountsA[17]*fcCountsB[8]*0.4565429;
	dSum += fcCountsA[17]*fcCountsB[9]*1.3142355;
	dSum += fcCountsA[17]*fcCountsB[10]*1.2689365;
	dSum += fcCountsA[17]*fcCountsB[11]*0.3690340;
	dSum += fcCountsA[17]*fcCountsB[12]*0.4430825;
	dSum += fcCountsA[17]*fcCountsB[13]*0.4667781;
	dSum += fcCountsA[17]*fcCountsB[14]*0.4200721;
	dSum += fcCountsA[17]*fcCountsB[15]*0.5652240;
	dSum += fcCountsA[17]*fcCountsB[16]*0.9809432;
	dSum += fcCountsA[17]*fcCountsB[17]*3.6921553;
	dSum += fcCountsA[17]*fcCountsB[18]*0.3744576;
	dSum += fcCountsA[17]*fcCountsB[19]*0.6580390;
	dSum += fcCountsA[18]*fcCountsB[0]*0.4165484;
	dSum += fcCountsA[18]*fcCountsB[1]*0.4499807;
	dSum += fcCountsA[18]*fcCountsB[2]*0.2321050;
	dSum += fcCountsA[18]*fcCountsB[3]*0.3743021;
	dSum += fcCountsA[18]*fcCountsB[4]*1.3743775;
	dSum += fcCountsA[18]*fcCountsB[5]*0.4216898;
	dSum += fcCountsA[18]*fcCountsB[6]*0.4440859;
	dSum += fcCountsA[18]*fcCountsB[7]*0.4088732;
	dSum += fcCountsA[18]*fcCountsB[8]*0.3589319;
	dSum += fcCountsA[18]*fcCountsB[9]*0.5680359;
	dSum += fcCountsA[18]*fcCountsB[10]*0.6103022;
	dSum += fcCountsA[18]*fcCountsB[11]*0.2777841;
	dSum += fcCountsA[18]*fcCountsB[12]*0.2818321;
	dSum += fcCountsA[18]*fcCountsB[13]*0.5093584;
	dSum += fcCountsA[18]*fcCountsB[14]*0.3951049;
	dSum += fcCountsA[18]*fcCountsB[15]*0.3853031;
	dSum += fcCountsA[18]*fcCountsB[16]*0.4309317;
	dSum += fcCountsA[18]*fcCountsB[17]*0.3744576;
	dSum += fcCountsA[18]*fcCountsB[18]*38.1077830;
	dSum += fcCountsA[18]*fcCountsB[19]*2.1098056;
	dSum += fcCountsA[19]*fcCountsB[0]*0.5426125;
	dSum += fcCountsA[19]*fcCountsB[1]*0.4342013;
	dSum += fcCountsA[19]*fcCountsB[2]*0.3456829;
	dSum += fcCountsA[19]*fcCountsB[3]*0.4964664;
	dSum += fcCountsA[19]*fcCountsB[4]*2.7693817;
	dSum += fcCountsA[19]*fcCountsB[5]*0.3487141;
	dSum += fcCountsA[19]*fcCountsB[6]*1.7979036;
	dSum += fcCountsA[19]*fcCountsB[7]*0.6303898;
	dSum += fcCountsA[19]*fcCountsB[8]*0.5321784;
	dSum += fcCountsA[19]*fcCountsB[9]*0.6920589;
	dSum += fcCountsA[19]*fcCountsB[10]*0.7083636;
	dSum += fcCountsA[19]*fcCountsB[11]*0.4860309;
	dSum += fcCountsA[19]*fcCountsB[12]*0.3635216;
	dSum += fcCountsA[19]*fcCountsB[13]*0.6110951;
	dSum += fcCountsA[19]*fcCountsB[14]*0.5559652;
	dSum += fcCountsA[19]*fcCountsB[15]*0.5575206;
	dSum += fcCountsA[19]*fcCountsB[16]*0.5731577;
	dSum += fcCountsA[19]*fcCountsB[17]*0.6580390;
	dSum += fcCountsA[19]*fcCountsB[18]*2.1098056;
	dSum += fcCountsA[19]*fcCountsB[19]*9.8322054;

	if (0.0 == dSum)
		return MINUS_INFINITY;

	return (SCORE) log(dSum);
	}

SCORE PWLAI::LAIScore(const FCOUNT fcCountsA[MAX_ALPHA],
  const FCOUNT fcCountsB[MAX_ALPHA])
	{
	double dSum = 0.0;

// Loop-unrolled double sum for log-average score computation (integer)
	dSum += fcCountsA[0]*fcCountsB[0]*4;
	dSum += fcCountsA[0]*fcCountsB[1]*1;
	dSum += fcCountsA[0]*fcCountsB[2]*1;
	dSum += fcCountsA[0]*fcCountsB[3]*1;
	dSum += fcCountsA[0]*fcCountsB[4]*0;
	dSum += fcCountsA[0]*fcCountsB[5]*1;
	dSum += fcCountsA[0]*fcCountsB[6]*1;
	dSum += fcCountsA[0]*fcCountsB[7]*1;
	dSum += fcCountsA[0]*fcCountsB[8]*1;
	dSum += fcCountsA[0]*fcCountsB[9]*1;
	dSum += fcCountsA[0]*fcCountsB[10]*1;
	dSum += fcCountsA[0]*fcCountsB[11]*1;
	dSum += fcCountsA[0]*fcCountsB[12]*1;
	dSum += fcCountsA[0]*fcCountsB[13]*1;
	dSum += fcCountsA[0]*fcCountsB[14]*1;
	dSum += fcCountsA[0]*fcCountsB[15]*1;
	dSum += fcCountsA[0]*fcCountsB[16]*1;
	dSum += fcCountsA[0]*fcCountsB[17]*1;
	dSum += fcCountsA[0]*fcCountsB[18]*0;
	dSum += fcCountsA[0]*fcCountsB[19]*1;
	dSum += fcCountsA[1]*fcCountsB[0]*1;
	dSum += fcCountsA[1]*fcCountsB[1]*20;
	dSum += fcCountsA[1]*fcCountsB[2]*0;
	dSum += fcCountsA[1]*fcCountsB[3]*0;
	dSum += fcCountsA[1]*fcCountsB[4]*0;
	dSum += fcCountsA[1]*fcCountsB[5]*0;
	dSum += fcCountsA[1]*fcCountsB[6]*0;
	dSum += fcCountsA[1]*fcCountsB[7]*1;
	dSum += fcCountsA[1]*fcCountsB[8]*0;
	dSum += fcCountsA[1]*fcCountsB[9]*1;
	dSum += fcCountsA[1]*fcCountsB[10]*1;
	dSum += fcCountsA[1]*fcCountsB[11]*0;
	dSum += fcCountsA[1]*fcCountsB[12]*0;
	dSum += fcCountsA[1]*fcCountsB[13]*0;
	dSum += fcCountsA[1]*fcCountsB[14]*0;
	dSum += fcCountsA[1]*fcCountsB[15]*1;
	dSum += fcCountsA[1]*fcCountsB[16]*1;
	dSum += fcCountsA[1]*fcCountsB[17]*1;
	dSum += fcCountsA[1]*fcCountsB[18]*0;
	dSum += fcCountsA[1]*fcCountsB[19]*0;
	dSum += fcCountsA[2]*fcCountsB[0]*1;
	dSum += fcCountsA[2]*fcCountsB[1]*0;
	dSum += fcCountsA[2]*fcCountsB[2]*7;
	dSum += fcCountsA[2]*fcCountsB[3]*2;
	dSum += fcCountsA[2]*fcCountsB[4]*0;
	dSum += fcCountsA[2]*fcCountsB[5]*1;
	dSum += fcCountsA[2]*fcCountsB[6]*1;
	dSum += fcCountsA[2]*fcCountsB[7]*0;
	dSum += fcCountsA[2]*fcCountsB[8]*1;
	dSum += fcCountsA[2]*fcCountsB[9]*0;
	dSum += fcCountsA[2]*fcCountsB[10]*0;
	dSum += fcCountsA[2]*fcCountsB[11]*2;
	dSum += fcCountsA[2]*fcCountsB[12]*1;
	dSum += fcCountsA[2]*fcCountsB[13]*1;
	dSum += fcCountsA[2]*fcCountsB[14]*1;
	dSum += fcCountsA[2]*fcCountsB[15]*1;
	dSum += fcCountsA[2]*fcCountsB[16]*1;
	dSum += fcCountsA[2]*fcCountsB[17]*0;
	dSum += fcCountsA[2]*fcCountsB[18]*0;
	dSum += fcCountsA[2]*fcCountsB[19]*0;
	dSum += fcCountsA[3]*fcCountsB[0]*1;
	dSum += fcCountsA[3]*fcCountsB[1]*0;
	dSum += fcCountsA[3]*fcCountsB[2]*2;
	dSum += fcCountsA[3]*fcCountsB[3]*5;
	dSum += fcCountsA[3]*fcCountsB[4]*0;
	dSum += fcCountsA[3]*fcCountsB[5]*0;
	dSum += fcCountsA[3]*fcCountsB[6]*1;
	dSum += fcCountsA[3]*fcCountsB[7]*0;
	dSum += fcCountsA[3]*fcCountsB[8]*1;
	dSum += fcCountsA[3]*fcCountsB[9]*0;
	dSum += fcCountsA[3]*fcCountsB[10]*1;
	dSum += fcCountsA[3]*fcCountsB[11]*1;
	dSum += fcCountsA[3]*fcCountsB[12]*1;
	dSum += fcCountsA[3]*fcCountsB[13]*2;
	dSum += fcCountsA[3]*fcCountsB[14]*1;
	dSum += fcCountsA[3]*fcCountsB[15]*1;
	dSum += fcCountsA[3]*fcCountsB[16]*1;
	dSum += fcCountsA[3]*fcCountsB[17]*0;
	dSum += fcCountsA[3]*fcCountsB[18]*0;
	dSum += fcCountsA[3]*fcCountsB[19]*0;
	dSum += fcCountsA[4]*fcCountsB[0]*0;
	dSum += fcCountsA[4]*fcCountsB[1]*0;
	dSum += fcCountsA[4]*fcCountsB[2]*0;
	dSum += fcCountsA[4]*fcCountsB[3]*0;
	dSum += fcCountsA[4]*fcCountsB[4]*8;
	dSum += fcCountsA[4]*fcCountsB[5]*0;
	dSum += fcCountsA[4]*fcCountsB[6]*1;
	dSum += fcCountsA[4]*fcCountsB[7]*1;
	dSum += fcCountsA[4]*fcCountsB[8]*0;
	dSum += fcCountsA[4]*fcCountsB[9]*1;
	dSum += fcCountsA[4]*fcCountsB[10]*1;
	dSum += fcCountsA[4]*fcCountsB[11]*0;
	dSum += fcCountsA[4]*fcCountsB[12]*0;
	dSum += fcCountsA[4]*fcCountsB[13]*0;
	dSum += fcCountsA[4]*fcCountsB[14]*0;
	dSum += fcCountsA[4]*fcCountsB[15]*0;
	dSum += fcCountsA[4]*fcCountsB[16]*0;
	dSum += fcCountsA[4]*fcCountsB[17]*1;
	dSum += fcCountsA[4]*fcCountsB[18]*1;
	dSum += fcCountsA[4]*fcCountsB[19]*3;
	dSum += fcCountsA[5]*fcCountsB[0]*1;
	dSum += fcCountsA[5]*fcCountsB[1]*0;
	dSum += fcCountsA[5]*fcCountsB[2]*1;
	dSum += fcCountsA[5]*fcCountsB[3]*0;
	dSum += fcCountsA[5]*fcCountsB[4]*0;
	dSum += fcCountsA[5]*fcCountsB[5]*7;
	dSum += fcCountsA[5]*fcCountsB[6]*0;
	dSum += fcCountsA[5]*fcCountsB[7]*0;
	dSum += fcCountsA[5]*fcCountsB[8]*1;
	dSum += fcCountsA[5]*fcCountsB[9]*0;
	dSum += fcCountsA[5]*fcCountsB[10]*0;
	dSum += fcCountsA[5]*fcCountsB[11]*1;
	dSum += fcCountsA[5]*fcCountsB[12]*0;
	dSum += fcCountsA[5]*fcCountsB[13]*1;
	dSum += fcCountsA[5]*fcCountsB[14]*0;
	dSum += fcCountsA[5]*fcCountsB[15]*1;
	dSum += fcCountsA[5]*fcCountsB[16]*1;
	dSum += fcCountsA[5]*fcCountsB[17]*0;
	dSum += fcCountsA[5]*fcCountsB[18]*0;
	dSum += fcCountsA[5]*fcCountsB[19]*0;
	dSum += fcCountsA[6]*fcCountsB[0]*1;
	dSum += fcCountsA[6]*fcCountsB[1]*0;
	dSum += fcCountsA[6]*fcCountsB[2]*1;
	dSum += fcCountsA[6]*fcCountsB[3]*1;
	dSum += fcCountsA[6]*fcCountsB[4]*1;
	dSum += fcCountsA[6]*fcCountsB[5]*0;
	dSum += fcCountsA[6]*fcCountsB[6]*14;
	dSum += fcCountsA[6]*fcCountsB[7]*0;
	dSum += fcCountsA[6]*fcCountsB[8]*1;
	dSum += fcCountsA[6]*fcCountsB[9]*0;
	dSum += fcCountsA[6]*fcCountsB[10]*1;
	dSum += fcCountsA[6]*fcCountsB[11]*1;
	dSum += fcCountsA[6]*fcCountsB[12]*0;
	dSum += fcCountsA[6]*fcCountsB[13]*1;
	dSum += fcCountsA[6]*fcCountsB[14]*1;
	dSum += fcCountsA[6]*fcCountsB[15]*1;
	dSum += fcCountsA[6]*fcCountsB[16]*1;
	dSum += fcCountsA[6]*fcCountsB[17]*0;
	dSum += fcCountsA[6]*fcCountsB[18]*0;
	dSum += fcCountsA[6]*fcCountsB[19]*2;
	dSum += fcCountsA[7]*fcCountsB[0]*1;
	dSum += fcCountsA[7]*fcCountsB[1]*1;
	dSum += fcCountsA[7]*fcCountsB[2]*0;
	dSum += fcCountsA[7]*fcCountsB[3]*0;
	dSum += fcCountsA[7]*fcCountsB[4]*1;
	dSum += fcCountsA[7]*fcCountsB[5]*0;
	dSum += fcCountsA[7]*fcCountsB[6]*0;
	dSum += fcCountsA[7]*fcCountsB[7]*4;
	dSum += fcCountsA[7]*fcCountsB[8]*0;
	dSum += fcCountsA[7]*fcCountsB[9]*2;
	dSum += fcCountsA[7]*fcCountsB[10]*1;
	dSum += fcCountsA[7]*fcCountsB[11]*0;
	dSum += fcCountsA[7]*fcCountsB[12]*0;
	dSum += fcCountsA[7]*fcCountsB[13]*0;
	dSum += fcCountsA[7]*fcCountsB[14]*0;
	dSum += fcCountsA[7]*fcCountsB[15]*0;
	dSum += fcCountsA[7]*fcCountsB[16]*1;
	dSum += fcCountsA[7]*fcCountsB[17]*2;
	dSum += fcCountsA[7]*fcCountsB[18]*0;
	dSum += fcCountsA[7]*fcCountsB[19]*1;
	dSum += fcCountsA[8]*fcCountsB[0]*1;
	dSum += fcCountsA[8]*fcCountsB[1]*0;
	dSum += fcCountsA[8]*fcCountsB[2]*1;
	dSum += fcCountsA[8]*fcCountsB[3]*1;
	dSum += fcCountsA[8]*fcCountsB[4]*0;
	dSum += fcCountsA[8]*fcCountsB[5]*1;
	dSum += fcCountsA[8]*fcCountsB[6]*1;
	dSum += fcCountsA[8]*fcCountsB[7]*0;
	dSum += fcCountsA[8]*fcCountsB[8]*5;
	dSum += fcCountsA[8]*fcCountsB[9]*0;
	dSum += fcCountsA[8]*fcCountsB[10]*1;
	dSum += fcCountsA[8]*fcCountsB[11]*1;
	dSum += fcCountsA[8]*fcCountsB[12]*1;
	dSum += fcCountsA[8]*fcCountsB[13]*2;
	dSum += fcCountsA[8]*fcCountsB[14]*2;
	dSum += fcCountsA[8]*fcCountsB[15]*1;
	dSum += fcCountsA[8]*fcCountsB[16]*1;
	dSum += fcCountsA[8]*fcCountsB[17]*0;
	dSum += fcCountsA[8]*fcCountsB[18]*0;
	dSum += fcCountsA[8]*fcCountsB[19]*1;
	dSum += fcCountsA[9]*fcCountsB[0]*1;
	dSum += fcCountsA[9]*fcCountsB[1]*1;
	dSum += fcCountsA[9]*fcCountsB[2]*0;
	dSum += fcCountsA[9]*fcCountsB[3]*0;
	dSum += fcCountsA[9]*fcCountsB[4]*1;
	dSum += fcCountsA[9]*fcCountsB[5]*0;
	dSum += fcCountsA[9]*fcCountsB[6]*0;
	dSum += fcCountsA[9]*fcCountsB[7]*2;
	dSum += fcCountsA[9]*fcCountsB[8]*0;
	dSum += fcCountsA[9]*fcCountsB[9]*4;
	dSum += fcCountsA[9]*fcCountsB[10]*2;
	dSum += fcCountsA[9]*fcCountsB[11]*0;
	dSum += fcCountsA[9]*fcCountsB[12]*0;
	dSum += fcCountsA[9]*fcCountsB[13]*0;
	dSum += fcCountsA[9]*fcCountsB[14]*0;
	dSum += fcCountsA[9]*fcCountsB[15]*0;
	dSum += fcCountsA[9]*fcCountsB[16]*1;
	dSum += fcCountsA[9]*fcCountsB[17]*1;
	dSum += fcCountsA[9]*fcCountsB[18]*1;
	dSum += fcCountsA[9]*fcCountsB[19]*1;
	dSum += fcCountsA[10]*fcCountsB[0]*1;
	dSum += fcCountsA[10]*fcCountsB[1]*1;
	dSum += fcCountsA[10]*fcCountsB[2]*0;
	dSum += fcCountsA[10]*fcCountsB[3]*1;
	dSum += fcCountsA[10]*fcCountsB[4]*1;
	dSum += fcCountsA[10]*fcCountsB[5]*0;
	dSum += fcCountsA[10]*fcCountsB[6]*1;
	dSum += fcCountsA[10]*fcCountsB[7]*1;
	dSum += fcCountsA[10]*fcCountsB[8]*1;
	dSum += fcCountsA[10]*fcCountsB[9]*2;
	dSum += fcCountsA[10]*fcCountsB[10]*6;
	dSum += fcCountsA[10]*fcCountsB[11]*0;
	dSum += fcCountsA[10]*fcCountsB[12]*0;
	dSum += fcCountsA[10]*fcCountsB[13]*1;
	dSum += fcCountsA[10]*fcCountsB[14]*1;
	dSum += fcCountsA[10]*fcCountsB[15]*1;
	dSum += fcCountsA[10]*fcCountsB[16]*1;
	dSum += fcCountsA[10]*fcCountsB[17]*1;
	dSum += fcCountsA[10]*fcCountsB[18]*1;
	dSum += fcCountsA[10]*fcCountsB[19]*1;
	dSum += fcCountsA[11]*fcCountsB[0]*1;
	dSum += fcCountsA[11]*fcCountsB[1]*0;
	dSum += fcCountsA[11]*fcCountsB[2]*2;
	dSum += fcCountsA[11]*fcCountsB[3]*1;
	dSum += fcCountsA[11]*fcCountsB[4]*0;
	dSum += fcCountsA[11]*fcCountsB[5]*1;
	dSum += fcCountsA[11]*fcCountsB[6]*1;
	dSum += fcCountsA[11]*fcCountsB[7]*0;
	dSum += fcCountsA[11]*fcCountsB[8]*1;
	dSum += fcCountsA[11]*fcCountsB[9]*0;
	dSum += fcCountsA[11]*fcCountsB[10]*0;
	dSum += fcCountsA[11]*fcCountsB[11]*7;
	dSum += fcCountsA[11]*fcCountsB[12]*0;
	dSum += fcCountsA[11]*fcCountsB[13]*1;
	dSum += fcCountsA[11]*fcCountsB[14]*1;
	dSum += fcCountsA[11]*fcCountsB[15]*1;
	dSum += fcCountsA[11]*fcCountsB[16]*1;
	dSum += fcCountsA[11]*fcCountsB[17]*0;
	dSum += fcCountsA[11]*fcCountsB[18]*0;
	dSum += fcCountsA[11]*fcCountsB[19]*0;
	dSum += fcCountsA[12]*fcCountsB[0]*1;
	dSum += fcCountsA[12]*fcCountsB[1]*0;
	dSum += fcCountsA[12]*fcCountsB[2]*1;
	dSum += fcCountsA[12]*fcCountsB[3]*1;
	dSum += fcCountsA[12]*fcCountsB[4]*0;
	dSum += fcCountsA[12]*fcCountsB[5]*0;
	dSum += fcCountsA[12]*fcCountsB[6]*0;
	dSum += fcCountsA[12]*fcCountsB[7]*0;
	dSum += fcCountsA[12]*fcCountsB[8]*1;
	dSum += fcCountsA[12]*fcCountsB[9]*0;
	dSum += fcCountsA[12]*fcCountsB[10]*0;
	dSum += fcCountsA[12]*fcCountsB[11]*0;
	dSum += fcCountsA[12]*fcCountsB[12]*13;
	dSum += fcCountsA[12]*fcCountsB[13]*1;
	dSum += fcCountsA[12]*fcCountsB[14]*0;
	dSum += fcCountsA[12]*fcCountsB[15]*1;
	dSum += fcCountsA[12]*fcCountsB[16]*1;
	dSum += fcCountsA[12]*fcCountsB[17]*0;
	dSum += fcCountsA[12]*fcCountsB[18]*0;
	dSum += fcCountsA[12]*fcCountsB[19]*0;
	dSum += fcCountsA[13]*fcCountsB[0]*1;
	dSum += fcCountsA[13]*fcCountsB[1]*0;
	dSum += fcCountsA[13]*fcCountsB[2]*1;
	dSum += fcCountsA[13]*fcCountsB[3]*2;
	dSum += fcCountsA[13]*fcCountsB[4]*0;
	dSum += fcCountsA[13]*fcCountsB[5]*1;
	dSum += fcCountsA[13]*fcCountsB[6]*1;
	dSum += fcCountsA[13]*fcCountsB[7]*0;
	dSum += fcCountsA[13]*fcCountsB[8]*2;
	dSum += fcCountsA[13]*fcCountsB[9]*0;
	dSum += fcCountsA[13]*fcCountsB[10]*1;
	dSum += fcCountsA[13]*fcCountsB[11]*1;
	dSum += fcCountsA[13]*fcCountsB[12]*1;
	dSum += fcCountsA[13]*fcCountsB[13]*6;
	dSum += fcCountsA[13]*fcCountsB[14]*1;
	dSum += fcCountsA[13]*fcCountsB[15]*1;
	dSum += fcCountsA[13]*fcCountsB[16]*1;
	dSum += fcCountsA[13]*fcCountsB[17]*0;
	dSum += fcCountsA[13]*fcCountsB[18]*1;
	dSum += fcCountsA[13]*fcCountsB[19]*1;
	dSum += fcCountsA[14]*fcCountsB[0]*1;
	dSum += fcCountsA[14]*fcCountsB[1]*0;
	dSum += fcCountsA[14]*fcCountsB[2]*1;
	dSum += fcCountsA[14]*fcCountsB[3]*1;
	dSum += fcCountsA[14]*fcCountsB[4]*0;
	dSum += fcCountsA[14]*fcCountsB[5]*0;
	dSum += fcCountsA[14]*fcCountsB[6]*1;
	dSum += fcCountsA[14]*fcCountsB[7]*0;
	dSum += fcCountsA[14]*fcCountsB[8]*2;
	dSum += fcCountsA[14]*fcCountsB[9]*0;
	dSum += fcCountsA[14]*fcCountsB[10]*1;
	dSum += fcCountsA[14]*fcCountsB[11]*1;
	dSum += fcCountsA[14]*fcCountsB[12]*0;
	dSum += fcCountsA[14]*fcCountsB[13]*1;
	dSum += fcCountsA[14]*fcCountsB[14]*7;
	dSum += fcCountsA[14]*fcCountsB[15]*1;
	dSum += fcCountsA[14]*fcCountsB[16]*1;
	dSum += fcCountsA[14]*fcCountsB[17]*0;
	dSum += fcCountsA[14]*fcCountsB[18]*0;
	dSum += fcCountsA[14]*fcCountsB[19]*1;
	dSum += fcCountsA[15]*fcCountsB[0]*1;
	dSum += fcCountsA[15]*fcCountsB[1]*1;
	dSum += fcCountsA[15]*fcCountsB[2]*1;
	dSum += fcCountsA[15]*fcCountsB[3]*1;
	dSum += fcCountsA[15]*fcCountsB[4]*0;
	dSum += fcCountsA[15]*fcCountsB[5]*1;
	dSum += fcCountsA[15]*fcCountsB[6]*1;
	dSum += fcCountsA[15]*fcCountsB[7]*0;
	dSum += fcCountsA[15]*fcCountsB[8]*1;
	dSum += fcCountsA[15]*fcCountsB[9]*0;
	dSum += fcCountsA[15]*fcCountsB[10]*1;
	dSum += fcCountsA[15]*fcCountsB[11]*1;
	dSum += fcCountsA[15]*fcCountsB[12]*1;
	dSum += fcCountsA[15]*fcCountsB[13]*1;
	dSum += fcCountsA[15]*fcCountsB[14]*1;
	dSum += fcCountsA[15]*fcCountsB[15]*4;
	dSum += fcCountsA[15]*fcCountsB[16]*2;
	dSum += fcCountsA[15]*fcCountsB[17]*1;
	dSum += fcCountsA[15]*fcCountsB[18]*0;
	dSum += fcCountsA[15]*fcCountsB[19]*1;
	dSum += fcCountsA[16]*fcCountsB[0]*1;
	dSum += fcCountsA[16]*fcCountsB[1]*1;
	dSum += fcCountsA[16]*fcCountsB[2]*1;
	dSum += fcCountsA[16]*fcCountsB[3]*1;
	dSum += fcCountsA[16]*fcCountsB[4]*0;
	dSum += fcCountsA[16]*fcCountsB[5]*1;
	dSum += fcCountsA[16]*fcCountsB[6]*1;
	dSum += fcCountsA[16]*fcCountsB[7]*1;
	dSum += fcCountsA[16]*fcCountsB[8]*1;
	dSum += fcCountsA[16]*fcCountsB[9]*1;
	dSum += fcCountsA[16]*fcCountsB[10]*1;
	dSum += fcCountsA[16]*fcCountsB[11]*1;
	dSum += fcCountsA[16]*fcCountsB[12]*1;
	dSum += fcCountsA[16]*fcCountsB[13]*1;
	dSum += fcCountsA[16]*fcCountsB[14]*1;
	dSum += fcCountsA[16]*fcCountsB[15]*2;
	dSum += fcCountsA[16]*fcCountsB[16]*5;
	dSum += fcCountsA[16]*fcCountsB[17]*1;
	dSum += fcCountsA[16]*fcCountsB[18]*0;
	dSum += fcCountsA[16]*fcCountsB[19]*1;
	dSum += fcCountsA[17]*fcCountsB[0]*1;
	dSum += fcCountsA[17]*fcCountsB[1]*1;
	dSum += fcCountsA[17]*fcCountsB[2]*0;
	dSum += fcCountsA[17]*fcCountsB[3]*0;
	dSum += fcCountsA[17]*fcCountsB[4]*1;
	dSum += fcCountsA[17]*fcCountsB[5]*0;
	dSum += fcCountsA[17]*fcCountsB[6]*0;
	dSum += fcCountsA[17]*fcCountsB[7]*2;
	dSum += fcCountsA[17]*fcCountsB[8]*0;
	dSum += fcCountsA[17]*fcCountsB[9]*1;
	dSum += fcCountsA[17]*fcCountsB[10]*1;
	dSum += fcCountsA[17]*fcCountsB[11]*0;
	dSum += fcCountsA[17]*fcCountsB[12]*0;
	dSum += fcCountsA[17]*fcCountsB[13]*0;
	dSum += fcCountsA[17]*fcCountsB[14]*0;
	dSum += fcCountsA[17]*fcCountsB[15]*1;
	dSum += fcCountsA[17]*fcCountsB[16]*1;
	dSum += fcCountsA[17]*fcCountsB[17]*4;
	dSum += fcCountsA[17]*fcCountsB[18]*0;
	dSum += fcCountsA[17]*fcCountsB[19]*1;
	dSum += fcCountsA[18]*fcCountsB[0]*0;
	dSum += fcCountsA[18]*fcCountsB[1]*0;
	dSum += fcCountsA[18]*fcCountsB[2]*0;
	dSum += fcCountsA[18]*fcCountsB[3]*0;
	dSum += fcCountsA[18]*fcCountsB[4]*1;
	dSum += fcCountsA[18]*fcCountsB[5]*0;
	dSum += fcCountsA[18]*fcCountsB[6]*0;
	dSum += fcCountsA[18]*fcCountsB[7]*0;
	dSum += fcCountsA[18]*fcCountsB[8]*0;
	dSum += fcCountsA[18]*fcCountsB[9]*1;
	dSum += fcCountsA[18]*fcCountsB[10]*1;
	dSum += fcCountsA[18]*fcCountsB[11]*0;
	dSum += fcCountsA[18]*fcCountsB[12]*0;
	dSum += fcCountsA[18]*fcCountsB[13]*1;
	dSum += fcCountsA[18]*fcCountsB[14]*0;
	dSum += fcCountsA[18]*fcCountsB[15]*0;
	dSum += fcCountsA[18]*fcCountsB[16]*0;
	dSum += fcCountsA[18]*fcCountsB[17]*0;
	dSum += fcCountsA[18]*fcCountsB[18]*38;
	dSum += fcCountsA[18]*fcCountsB[19]*2;
	dSum += fcCountsA[19]*fcCountsB[0]*1;
	dSum += fcCountsA[19]*fcCountsB[1]*0;
	dSum += fcCountsA[19]*fcCountsB[2]*0;
	dSum += fcCountsA[19]*fcCountsB[3]*0;
	dSum += fcCountsA[19]*fcCountsB[4]*3;
	dSum += fcCountsA[19]*fcCountsB[5]*0;
	dSum += fcCountsA[19]*fcCountsB[6]*2;
	dSum += fcCountsA[19]*fcCountsB[7]*1;
	dSum += fcCountsA[19]*fcCountsB[8]*1;
	dSum += fcCountsA[19]*fcCountsB[9]*1;
	dSum += fcCountsA[19]*fcCountsB[10]*1;
	dSum += fcCountsA[19]*fcCountsB[11]*0;
	dSum += fcCountsA[19]*fcCountsB[12]*0;
	dSum += fcCountsA[19]*fcCountsB[13]*1;
	dSum += fcCountsA[19]*fcCountsB[14]*1;
	dSum += fcCountsA[19]*fcCountsB[15]*1;
	dSum += fcCountsA[19]*fcCountsB[16]*1;
	dSum += fcCountsA[19]*fcCountsB[17]*1;
	dSum += fcCountsA[19]*fcCountsB[18]*2;
	dSum += fcCountsA[19]*fcCountsB[19]*10;
	if (0.0 == dSum)
		return MINUS_INFINITY;

	return (SCORE) log(dSum);
	}
